#!/usr/bin/python
#
# s1 -- csound-based sampler
# Copyright (c) 1999 Eric S. Tiedemann
# Released under the GPL (version 2 or later; see file COPYING)

### these comments by PW!

import sys
import os
import string
import tempfile

ourorc = '''

; s1 sampler engine

	sr=44100
	kr=441
	ksmps=100
	nchnls=1

	instr 1
inum  	notnum
ivel  	veloc
ifqc  	cpsmidi
iamp  	ampmidi inum*500
ifn	table inum-1, 1
	print ifn

asig	loscil iamp, 100, ifn
	out asig
	endin
'''

s1f = open(sys.argv[1]) ### open the keymap file
s1i = s1f.read() ### read it into a string
s1f.close()

s1 = eval(s1i) ### evaluating the s1i string makes s1 into the dict
               ### represented by the string.
               ### key: value represent midi_key : soundfile

nf = 2       ### start with f2.
sfiles = {}  ### this will be of the form f_table : soundfile
             ### -- used to make ftables in the sco for each soundfile
fns = {}     ### this will be of the form soundfile : f_table
             ###  -- used to make a keymap table for all 128 midi keys.

for k in s1.keys(): ### for each midi key in the dict...
  f = s1[k]           ### f is the associated soundfile;
  if (not fns.has_key(f)): ### is this soundfile in fns? If not:
    sfiles[nf] = f  ### put soundfile in sfiles, with value = ftable       
    fns[f] = nf     ### put ftable in fns, with value =  soundfile
    nf = nf + 1     ### next ftable.

tabf = tempfile.mktemp()

tab = open(tabf, 'w')

### Now we make a text file in which each line contains one number.
### The numbers correspond to ftables.
### There are 128 lines corresponding to the midi keyboard.
### The csound sco will read this file into table f1 by using gen 23.

# prints the table 
for x in range(1,129):
  try: f = s1[x]
  except: f = s1.values()[0]
  tab.write(`fns[f]` + "\n")

tab.close()

orcf = tempfile.mktemp()
orc = open(orcf, 'w')
orc.write(ourorc)
orc.close()

scof = tempfile.mktemp()

sco = open(scof, 'w')

# prints the .sco file
sco.write(('f1 0 128 -23 "%s"' % tabf) + "\n") ### use gen 23 to
                                               ### make the keymap table
                                               ### from our stored file.
### Now make ftables for each soundfile.
for i in range(2,nf):
  sco.write("f%d 0 0 1 \"%s\" 0 0 0\n" % (i, sfiles[i]))
sco.write("i0 0 36000\n") ### dummy note to make performance last 36000 sec.

sco.close()

### Now run csound.
os.system("csound " + string.join(sys.argv[2:], ' ') + " %s %s" % (orcf, scof))

### Delete temp files.
os.unlink(orcf)
os.unlink(tabf)
os.unlink(scof)

sys.exit(0)
